'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DtsCreator = undefined;

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _isThere = require('is-there');

var _isThere2 = _interopRequireDefault(_isThere);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _camelcase = require('camelcase');

var _camelcase2 = _interopRequireDefault(_camelcase);

var _tokenValidator = require('./tokenValidator');

var _fileSystemLoader = require('./fileSystemLoader');

var _fileSystemLoader2 = _interopRequireDefault(_fileSystemLoader);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let validator = new _tokenValidator.TokenValidator();

function removeExtension(filePath) {
  const ext = _path2.default.extname(filePath);
  return filePath.replace(new RegExp(ext + '$'), '');
}

class DtsContent {
  constructor({
    dropExtension,
    rootDir,
    searchDir,
    outDir,
    rInputPath,
    rawTokenList,
    resultList,
    messageList
  }) {
    this.dropExtension = dropExtension;
    this.rootDir = rootDir;
    this.searchDir = searchDir;
    this.outDir = outDir;
    this.rInputPath = rInputPath;
    this.rawTokenList = rawTokenList;
    this.resultList = resultList;
    this.messageList = messageList;
  }

  get contents() {
    return this.resultList;
  }

  get formatted() {
    if (!this.resultList || !this.resultList.length) return '';
    return this.resultList.join(_os2.default.EOL);
  }

  get tokens() {
    return this.rawTokenList;
  }

  get outputFilePath() {
    const outputFileName = this.dropExtension ? removeExtension(this.rInputPath) : this.rInputPath;
    return _path2.default.join(this.rootDir, this.outDir, outputFileName + '.d.ts');
  }

  get inputFilePath() {
    return _path2.default.join(this.rootDir, this.searchDir, this.rInputPath);
  }

  writeFile() {
    var outPathDir = _path2.default.dirname(this.outputFilePath);
    if (!(0, _isThere2.default)(outPathDir)) {
      _mkdirp2.default.sync(outPathDir);
    }
    return new Promise((resolve, reject) => {
      _fs2.default.writeFile(this.outputFilePath, this.formatted + _os2.default.EOL, 'utf8', err => {
        if (err) {
          reject(err);
        } else {
          resolve(this);
        }
      });
    });
  }
}

class DtsCreator {
  constructor(options) {
    if (!options) options = {};
    this.rootDir = options.rootDir || _process2.default.cwd();
    this.searchDir = options.searchDir || '';
    this.outDir = options.outDir || this.searchDir;
    this.loader = new _fileSystemLoader2.default(this.rootDir);
    this.inputDirectory = _path2.default.join(this.rootDir, this.searchDir);
    this.outputDirectory = _path2.default.join(this.rootDir, this.outDir);
    this.camelCase = options.camelCase;
    this.dropExtension = !!options.dropExtension;
  }

  create(filePath, initialContents, clearCache = false) {
    return new Promise((resolve, reject) => {
      var rInputPath;
      if (_path2.default.isAbsolute(filePath)) {
        rInputPath = _path2.default.relative(this.inputDirectory, filePath);
      } else {
        rInputPath = _path2.default.relative(this.inputDirectory, _path2.default.join(_process2.default.cwd(), filePath));
      }
      if (clearCache) {
        this.loader.tokensByFile = {};
      }
      this.loader.fetch(filePath, "/", undefined, initialContents).then(res => {
        if (res) {
          var tokens = res;
          var keys = Object.keys(tokens);
          var validKeys = [],
              invalidKeys = [];
          var messageList = [];

          var convertKey = this.getConvertKeyMethod(this.camelCase);

          keys.forEach(key => {
            const convertedKey = convertKey(key);
            var ret = validator.validate(convertedKey);
            if (ret.isValid) {
              validKeys.push(convertedKey);
            } else {
              messageList.push(ret.message);
            }
          });

          var result = validKeys.map(k => 'export const ' + k + ': string;');

          var content = new DtsContent({
            dropExtension: this.dropExtension,
            rootDir: this.rootDir,
            searchDir: this.searchDir,
            outDir: this.outDir,
            rInputPath,
            rawTokenList: keys,
            resultList: result,
            messageList
          });

          resolve(content);
        } else {
          reject(res);
        }
      }).catch(err => reject(err));
    });
  }

  getConvertKeyMethod(camelCaseOption) {
    switch (camelCaseOption) {
      case true:
        return _camelcase2.default;
      case 'dashes':
        return this.dashesCamelCase;
      default:
        return key => key;
    }
  }

  /**
   * Replaces only the dashes and leaves the rest as-is.
   *
   * Mirrors the behaviour of the css-loader:
   * https://github.com/webpack-contrib/css-loader/blob/1fee60147b9dba9480c9385e0f4e581928ab9af9/lib/compile-exports.js#L3-L7
   */
  dashesCamelCase(str) {
    return str.replace(/-+(\w)/g, function (match, firstLetter) {
      return firstLetter.toUpperCase();
    });
  }

}
exports.DtsCreator = DtsCreator;